=== modified file 'NEWS'
--- NEWS	2005-09-16 14:40:41 +0000
+++ NEWS	2005-09-16 14:57:59 +0000
@@ -8,6 +8,9 @@
       get added when you add a file in the directory.  Contributed by
       Michael Ellerman.
 
+    * Ignore .DS_Store (contains Mac metadata) by default.  Patch from
+      Nir Soffer.
+
 
   INTERNALS:
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-09-16 14:40:41 +0000
+++ bzrlib/__init__.py	2005-09-16 14:57:59 +0000
@@ -28,7 +28,8 @@
                   'BitKeeper',
                   '.git',
                   'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*']
+                  '.del-*',
+                  '.DS_Store',]
 
 IGNORE_FILENAME = ".bzrignore"
 
@@ -48,7 +49,7 @@
     
     try:
         import os
-        branch = Branch(os.path.dirname(__path__[0]))
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-16 14:40:41 +0000
+++ bzrlib/add.py	2005-09-16 14:57:59 +0000
@@ -64,7 +64,7 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch(file_list[0], find_root=True)
+    b = Branch.open_containing(file_list[0])
     return smart_add_branch(b, file_list, recurse, reporter)
 
         
@@ -112,7 +112,7 @@
 
         if kind == 'directory':
             try:
-                sub_branch = Branch(af, find_root=False)
+                sub_branch = Branch.open(af)
                 sub_tree = True
             except NotBranchError:
                 sub_tree = False

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-16 14:40:41 +0000
+++ bzrlib/branch.py	2005-09-16 14:57:59 +0000
@@ -32,6 +32,7 @@
 from bzrlib.tree import EmptyTree, RevisionTree
 import bzrlib.xml
 import bzrlib.ui
+import bzrlib.transport
 
 
 
@@ -43,26 +44,10 @@
 # repeatedly to calculate deltas.  We could perhaps have a weakref
 # cache in memory to make this faster.
 
-# TODO: please move the revision-string syntax stuff out of the branch
-# object; it's clutter
-
-
-def find_branch(f, **args):
-    from bzrlib.transport import transport
-    from bzrlib.transport.local import LocalTransport
-    t = transport(f)
-    # FIXME: This is a hack around transport so that
-    #        We can search the local directories for
-    #        a branch root.
-    if args.has_key('init') and args['init']:
-        # Don't search if we are init-ing
-        return Branch(t, **args)
-    if isinstance(t, LocalTransport):
-        root = find_branch_root(f)
-        if root != f:
-            t = transport(root)
-    return Branch(t, **args)
-
+def find_branch(*ignored, **ignored_too):
+    # XXX: leave this here for about one release, then remove it
+    raise NotImplementedError('find_branch() is not supported anymore, '
+                              'please use one of the new branch constructors')
 def _relpath(base, path):
     """Return path relative to base, or raise exception.
 
@@ -90,36 +75,24 @@
     return os.sep.join(s)
         
 
-def find_branch_root(f=None):
-    """Find the branch root enclosing f, or pwd.
-
-    f may be a filename or a URL.
-
-    It is not necessary that f exists.
+def find_branch_root(t):
+    """Find the branch root enclosing the transport's base.
+
+    t is a Transport object.
+
+    It is not necessary that the base of t exists.
 
     Basically we keep looking up until we find the control directory or
     run into the root.  If there isn't one, raises NotBranchError.
     """
-    if f == None:
-        f = os.getcwd()
-    else:
-        f = os.path.realpath(f)
-    if not os.path.exists(f):
-        raise BzrError('%r does not exist' % f)
-        
-
-    orig_f = f
-
     while True:
-        if os.path.exists(os.path.join(f, bzrlib.BZRDIR)):
-            return f
-        head, tail = os.path.split(f)
-        if head == f:
+        if t.has(bzrlib.BZRDIR):
+            return t
+        new_t = t.clone('..')
+        if new_t.base == t.base:
             # reached the root, whatever that may be
             raise NotBranchError('%s is not in a branch' % orig_f)
-        f = head
-
-
+        t = new_t
 
 
 ######################################################################
@@ -129,7 +102,45 @@
     """Branch holding a history of revisions.
 
     base
-        Base directory of the branch.
+        Base directory/url of the branch.
+    """
+    base = None
+
+    def __init__(self, *ignored, **ignored_too):
+        raise NotImplementedError('The Branch class is abstract')
+
+    @staticmethod
+    def open(base):
+        """Open an existing branch, rooted at 'base' (url)"""
+        t = bzrlib.transport.transport(base)
+        return LocalBranch(t)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch, containing url (search upwards for the root)
+        """
+        t = bzrlib.transport.transport(base)
+        found_t = find_branch_root(t)
+        return LocalBranch(t)
+
+    @staticmethod
+    def initialize(base):
+        """Create a new branch, rooted at 'base' (url)"""
+        t = bzrlib.transport.transport(base)
+        return LocalBranch(t, init=True)
+
+    def setup_caching(self, cache_root):
+        """Subclasses that care about caching should override this, and set
+        up cached stores located under cache_root.
+        """
+
+
+class LocalBranch(Branch):
+    """A branch stored in the actual filesystem.
+
+    Note that it's "local" in the context of the filesystem; it doesn't
+    really matter if it's on an nfs/smb/afs/coda/... share, as long as
+    it's writable, and can be accessed via the normal filesystem API.
 
     _lock_mode
         None, or 'r' or 'w'
@@ -141,16 +152,13 @@
     _lock
         Lock object from bzrlib.lock.
     """
-    base = None
+    # We actually expect this class to be somewhat short-lived; part of its
+    # purpose is to try to isolate what bits of the branch logic are tied to
+    # filesystem access, so that in a later step, we can extricate them to
+    # a separarte ("storage") class.
     _lock_mode = None
     _lock_count = None
     _lock = None
-    cache_root = None
-    
-    # Map some sort of prefix into a namespace
-    # stuff like "revno:10", "revid:", etc.
-    # This should match a prefix with a function which accepts
-    REVISION_NAMESPACES = {}
 
     def __init__(self, transport, init=False):
         """Create new branch object at a particular location.
@@ -527,7 +535,7 @@
         """Print `file` to stdout."""
         self.lock_read()
         try:
-            tree = self.revision_tree(self.lookup_revision(revno))
+            tree = self.revision_tree(self.get_rev_id(revno))
             # use inventory as it was in that revision
             file_id = tree.inventory.path2id(file)
             if not file_id:
@@ -911,7 +919,7 @@
         if stop_revision is None:
             other_revision = other.last_patch()
         else:
-            other_revision = other.lookup_revision(stop_revision)
+            other_revision = other.get_rev_id(stop_revision)
         count = greedy_fetch(self, other, other_revision, pb)[0]
         try:
             revision_ids = self.missing_revisions(other, stop_revision)
@@ -994,12 +1002,6 @@
         commit(self, *args, **kw)
         
 
-    def lookup_revision(self, revision):
-        """Return the revision identifier for a given revision information."""
-        revno, info = self._get_revision_info(revision)
-        return info
-
-
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
         history = self.revision_history()
@@ -1009,19 +1011,6 @@
             raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
 
-    def get_revision_info(self, revision):
-        """Return (revno, revision id) for revision identifier.
-
-        revision can be an integer, in which case it is assumed to be revno (though
-            this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something like
-            'date:' or 'revid:' etc.
-        """
-        revno, rev_id = self._get_revision_info(revision)
-        if revno is None:
-            raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
     def get_rev_id(self, revno, history=None):
         """Find the revision id of the specified revno."""
         if revno == 0:
@@ -1032,191 +1021,6 @@
             raise bzrlib.errors.NoSuchRevision(self, revno)
         return history[revno - 1]
 
-    def _get_revision_info(self, revision):
-        """Return (revno, revision id) for revision specifier.
-
-        revision can be an integer, in which case it is assumed to be revno
-        (though this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something
-        like 'date:' or 'revid:' etc.
-
-        A revid is always returned.  If it is None, the specifier referred to
-        the null revision.  If the revid does not occur in the revision
-        history, revno will be None.
-        """
-        
-        if revision is None:
-            return 0, None
-        revno = None
-        try:# Convert to int if possible
-            revision = int(revision)
-        except ValueError:
-            pass
-        revs = self.revision_history()
-        if isinstance(revision, int):
-            if revision < 0:
-                revno = len(revs) + revision + 1
-            else:
-                revno = revision
-            rev_id = self.get_rev_id(revno, revs)
-        elif isinstance(revision, basestring):
-            for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
-                if revision.startswith(prefix):
-                    result = func(self, revs, revision)
-                    if len(result) > 1:
-                        revno, rev_id = result
-                    else:
-                        revno = result[0]
-                        rev_id = self.get_rev_id(revno, revs)
-                    break
-            else:
-                raise BzrError('No namespace registered for string: %r' %
-                               revision)
-        else:
-            raise TypeError('Unhandled revision type %s' % revision)
-
-        if revno is None:
-            if rev_id is None:
-                raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
-    def _namespace_revno(self, revs, revision):
-        """Lookup a revision by revision number"""
-        assert revision.startswith('revno:')
-        try:
-            return (int(revision[6:]),)
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revno:'] = _namespace_revno
-
-    def _namespace_revid(self, revs, revision):
-        assert revision.startswith('revid:')
-        rev_id = revision[len('revid:'):]
-        try:
-            return revs.index(rev_id) + 1, rev_id
-        except ValueError:
-            return None, rev_id
-    REVISION_NAMESPACES['revid:'] = _namespace_revid
-
-    def _namespace_last(self, revs, revision):
-        assert revision.startswith('last:')
-        try:
-            offset = int(revision[5:])
-        except ValueError:
-            return (None,)
-        else:
-            if offset <= 0:
-                raise BzrError('You must supply a positive value for --revision last:XXX')
-            return (len(revs) - offset + 1,)
-    REVISION_NAMESPACES['last:'] = _namespace_last
-
-    def _namespace_tag(self, revs, revision):
-        assert revision.startswith('tag:')
-        raise BzrError('tag: namespace registered, but not implemented.')
-    REVISION_NAMESPACES['tag:'] = _namespace_tag
-
-    def _namespace_date(self, revs, revision):
-        assert revision.startswith('date:')
-        import datetime
-        # Spec for date revisions:
-        #   date:value
-        #   value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
-        #   it can also start with a '+/-/='. '+' says match the first
-        #   entry after the given date. '-' is match the first entry before the date
-        #   '=' is match the first entry after, but still on the given date.
-        #
-        #   +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
-        #   -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
-        #   =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
-        #       May 13th, 2005 at 0:00
-        #
-        #   So the proper way of saying 'give me all entries for today' is:
-        #       -r {date:+today}:{date:-tomorrow}
-        #   The default is '=' when not supplied
-        val = revision[5:]
-        match_style = '='
-        if val[:1] in ('+', '-', '='):
-            match_style = val[:1]
-            val = val[1:]
-
-        today = datetime.datetime.today().replace(hour=0,minute=0,second=0,microsecond=0)
-        if val.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif val.lower() == 'today':
-            dt = today
-        elif val.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
-        else:
-            import re
-            # This should be done outside the function to avoid recompiling it.
-            _date_re = re.compile(
-                    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-                    r'(,|T)?\s*'
-                    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-                )
-            m = _date_re.match(val)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise BzrError('Invalid revision date %r' % revision)
-
-            if m.group('date'):
-                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
-            else:
-                year, month, day = today.year, today.month, today.day
-            if m.group('time'):
-                hour = int(m.group('hour'))
-                minute = int(m.group('minute'))
-                if m.group('second'):
-                    second = int(m.group('second'))
-                else:
-                    second = 0
-            else:
-                hour, minute, second = 0,0,0
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                    hour=hour, minute=minute, second=second)
-        first = dt
-        last = None
-        reversed = False
-        if match_style == '-':
-            reversed = True
-        elif match_style == '=':
-            last = dt + datetime.timedelta(days=1)
-
-        if reversed:
-            for i in range(len(revs)-1, -1, -1):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first >= dt and (last is None or dt >= last):
-                    return (i+1,)
-        else:
-            for i in range(len(revs)):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first <= dt and (last is None or dt <= last):
-                    return (i+1,)
-    REVISION_NAMESPACES['date:'] = _namespace_date
-
-
-    def _namespace_ancestor(self, revs, revision):
-        from revision import common_ancestor, MultipleRevisionSources
-        other_branch = find_branch(_trim_namespace('ancestor', revision))
-        revision_a = self.last_patch()
-        revision_b = other_branch.last_patch()
-        for r, b in ((revision_a, self), (revision_b, other_branch)):
-            if r is None:
-                raise bzrlib.errors.NoCommits(b)
-        revision_source = MultipleRevisionSources(self, other_branch)
-        result = common_ancestor(revision_a, revision_b, revision_source)
-        try:
-            revno = self.revision_id_to_revno(result)
-        except bzrlib.errors.NoSuchRevision:
-            revno = None
-        return revno,result
-        
-
-    REVISION_NAMESPACES['ancestor:'] = _namespace_ancestor
 
     def revision_tree(self, revision_id):
         """Return Tree for a revision on this branch.
@@ -1503,9 +1307,10 @@
             raise InvalidRevisionNumber(revno)
         
         
-
-
-class ScratchBranch(Branch):
+        
+
+
+class ScratchBranch(LocalBranch):
     """Special test class: a branch that cleans up after itself.
 
     >>> b = ScratchBranch()
@@ -1528,7 +1333,7 @@
         if base is None:
             base = mkdtemp()
             init = True
-        Branch.__init__(self, base, init=init)
+        LocalBranch.__init__(self, base, init=init)
         for d in dirs:
             self._transport.mkdir(d)
             
@@ -1640,23 +1445,19 @@
         The name of a local directory that exists but is empty.
     """
     from bzrlib.merge import merge
+    from bzrlib.revisionspec import RevisionSpec
 
     assert isinstance(branch_from, Branch)
     assert isinstance(to_location, basestring)
     
-    br_to = Branch(to_location, init=True)
+    br_to = Branch.initialize(to_location)
     br_to.set_root_id(branch_from.get_root_id())
     if revision is None:
         revno = branch_from.revno()
     else:
-        revno, rev_id = branch_from.get_revision_info(revision)
+        revno, rev_id = RevisionSpec(revision).in_history(branch_from)
     br_to.update_revisions(branch_from, stop_revision=revno)
     merge((to_location, -1), (to_location, 0), this_dir=to_location,
           check_clean=False, ignore_zero=True)
     br_to.set_parent(branch_from.base)
     return br_to
-
-def _trim_namespace(namespace, spec):
-    full_namespace = namespace + ':'
-    assert spec.startswith(full_namespace)
-    return spec[len(full_namespace):]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-16 14:40:41 +0000
+++ bzrlib/builtins.py	2005-09-16 14:57:59 +0000
@@ -22,7 +22,7 @@
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
-from bzrlib.branch import find_branch
+from bzrlib.branch import Branch
 from bzrlib import BZRDIR
 from bzrlib.commands import Command
 
@@ -72,14 +72,14 @@
     
     def run(self, all=False, show_ids=False, file_list=None):
         if file_list:
-            b = find_branch(file_list[0])
+            b = Branch.open_containing(file_list[0])
             file_list = [b.relpath(x) for x in file_list]
             # special case: only one path was given and it's the root
             # of the branch
             if file_list == ['']:
                 file_list = None
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
             
         from bzrlib.status import show_status
         show_status(b, show_unchanged=all, show_ids=show_ids,
@@ -93,7 +93,7 @@
     takes_args = ['revision_id']
     
     def run(self, revision_id):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         sys.stdout.write(b.get_revision_xml_file(revision_id).read())
 
 
@@ -102,7 +102,7 @@
 
     This is equal to the number of revisions on this branch."""
     def run(self):
-        print find_branch('.').revno()
+        print Branch.open_containing('.').revno()
 
 
 class cmd_revision_info(Command):
@@ -111,21 +111,21 @@
     hidden = True
     takes_args = ['revision_info*']
     takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
+    def run(self, revision=None, revision_info_list=()):
+        from bzrlib.revisionspec import RevisionSpec
 
         revs = []
         if revision is not None:
             revs.extend(revision)
-        if revision_info_list is not None:
-            revs.extend(revision_info_list)
+        for rev in revision_info_list:
+            revs.append(RevisionSpec(revision_info_list))
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
 
         for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
+            print '%4d %s' % rev.in_history(b)
 
     
 class cmd_add(Command):
@@ -174,7 +174,7 @@
         for d in dir_list:
             os.mkdir(d)
             if not b:
-                b = find_branch(d)
+                b = Branch.open_containing(d)
             b.add([d])
             print 'added', d
 
@@ -185,7 +185,7 @@
     hidden = True
     
     def run(self, filename):
-        print find_branch(filename).relpath(filename)
+        print Branch.open_containing(filename).relpath(filename)
 
 
 
@@ -194,14 +194,14 @@
     takes_options = ['revision', 'show-ids']
     
     def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
+        b = Branch.open_containing('.')
+        if revision is None:
             inv = b.read_working_inventory()
         else:
             if len(revision) > 1:
                 raise BzrCommandError('bzr inventory --revision takes'
                     ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
+            inv = b.get_revision_inventory(revision[0].in_history(b).rev_id)
 
         for path, entry in inv.entries():
             if show_ids:
@@ -220,7 +220,7 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
 
         # TODO: glob expansion on windows?
         b.move([b.relpath(s) for s in source_list], b.relpath(dest))
@@ -243,7 +243,7 @@
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         b.rename_one(b.relpath(from_name), b.relpath(to_name))
 
 
@@ -265,7 +265,7 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = find_branch(names_list[0])
+        b = Branch.open_containing(names_list[0])
 
         rel_names = [b.relpath(x) for x in names_list]
         
@@ -305,7 +305,7 @@
         from shutil import rmtree
         import errno
         
-        br_to = find_branch('.')
+        br_to = Branch.open_containing('.')
         stored_loc = br_to.get_parent()
         if location is None:
             if stored_loc is None:
@@ -314,13 +314,14 @@
                 print "Using last location: %s" % stored_loc
                 location = stored_loc
         cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
+        from bzrlib.errors import DivergedBranches
+        br_from = Branch.open_containing(location)
         location = br_from.base
         old_revno = br_to.revno()
         try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
+            from bzrlib.errors import DivergedBranches
+            br_from = Branch.open(location)
+            br_from.setup_caching(cache_root)
             location = br_from.base
             old_revno = br_to.revno()
             try:
@@ -351,7 +352,7 @@
     aliases = ['get', 'clone']
 
     def run(self, from_location, to_location=None, revision=None):
-        from bzrlib.branch import copy_branch, find_cached_branch
+        from bzrlib.branch import copy_branch
         import tempfile
         import errno
         from shutil import rmtree
@@ -363,13 +364,14 @@
                 raise BzrCommandError(
                     'bzr branch --revision takes exactly 1 revision value')
             try:
-                br_from = find_cached_branch(from_location, cache_root)
+                br_from = Branch.open(from_location)
             except OSError, e:
                 if e.errno == errno.ENOENT:
                     raise BzrCommandError('Source location "%s" does not'
                                           ' exist.' % to_location)
                 else:
                     raise
+            br_from.setup_caching(cache_root)
             if to_location is None:
                 to_location = os.path.basename(from_location.rstrip("/\\"))
             try:
@@ -403,7 +405,7 @@
     takes_args = ['dir?']
 
     def run(self, dir='.'):
-        b = find_branch(dir)
+        b = Branch.open_containing(dir)
         old_inv = b.basis_tree().inventory
         new_inv = b.read_working_inventory()
 
@@ -420,7 +422,7 @@
     def run(self, branch=None):
         import info
 
-        b = find_branch(branch)
+        b = Branch.open_containing(branch)
         info.show_info(b)
 
 
@@ -434,7 +436,7 @@
     takes_options = ['verbose']
     
     def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
+        b = Branch.open_containing(file_list[0])
         b.remove([b.relpath(f) for f in file_list], verbose=verbose)
 
 
@@ -448,7 +450,7 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         i = b.inventory.path2id(b.relpath(filename))
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
@@ -464,7 +466,7 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         inv = b.inventory
         fid = inv.path2id(b.relpath(filename))
         if fid == None:
@@ -477,14 +479,14 @@
     """Display list of revision ids on this branch."""
     hidden = True
     def run(self):
-        for patchid in find_branch('.').revision_history():
+        for patchid in Branch.open_containing('.').revision_history():
             print patchid
 
 
 class cmd_directories(Command):
     """Display list of versioned directories in this branch."""
     def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
+        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
             if name == '':
                 print '.'
             else:
@@ -506,7 +508,7 @@
     """
     def run(self):
         from bzrlib.branch import Branch
-        Branch('.', init=True)
+        Branch.initialize('.')
 
 
 class cmd_diff(Command):
@@ -544,13 +546,13 @@
         from bzrlib.diff import show_diff
 
         if file_list:
-            b = find_branch(file_list[0])
+            b = Branch.open_containing(file_list[0])
             file_list = [b.relpath(f) for f in file_list]
             if file_list == ['']:
                 # just pointing to top-of-tree
                 file_list = None
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
 
         if revision is not None:
             if len(revision) == 1:
@@ -575,7 +577,7 @@
     TODO: Show files deleted since a previous revision, or between two revisions.
     """
     def run(self, show_ids=False):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         old = b.basis_tree()
         new = b.working_tree()
 
@@ -598,7 +600,7 @@
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         td = compare_trees(b.basis_tree(), b.working_tree())
 
         for path, id, kind in td.modified:
@@ -610,7 +612,7 @@
     """List files added in working tree."""
     hidden = True
     def run(self):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         wt = b.working_tree()
         basis_inv = b.basis_tree().inventory
         inv = wt.inventory
@@ -632,7 +634,7 @@
     takes_args = ['filename?']
     def run(self, filename=None):
         """Print the branch root."""
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         print b.base
 
 
@@ -668,24 +670,24 @@
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = find_branch(filename)
+            b = Branch.open_containing(filename)
             fp = b.relpath(filename)
             if fp:
                 file_id = b.read_working_inventory().path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
             file_id = None
 
         if revision is None:
             rev1 = None
             rev2 = None
         elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
+            rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
+            rev1 = revision[0].in_history(b).revno
+            rev2 = revision[1].in_history(b).revno
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 
@@ -727,7 +729,7 @@
     hidden = True
     takes_args = ["filename"]
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         inv = b.read_working_inventory()
         file_id = inv.path2id(b.relpath(filename))
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
@@ -741,11 +743,11 @@
     """
     hidden = True
     def run(self, revision=None, verbose=False):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if revision == None:
             tree = b.working_tree()
         else:
-            tree = b.revision_tree(b.lookup_revision(revision))
+            tree = b.revision_tree(revision.in_history(b).rev_id)
 
         for fp, fc, kind, fid in tree.list_files():
             if verbose:
@@ -766,7 +768,7 @@
     """List unknown files."""
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
+        for f in Branch.open_containing('.').unknowns():
             print quotefn(f)
 
 
@@ -794,7 +796,7 @@
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         ifn = b.abspath('.bzrignore')
 
         if os.path.exists(ifn):
@@ -834,7 +836,7 @@
 
     See also: bzr ignore"""
     def run(self):
-        tree = find_branch('.').working_tree()
+        tree = Branch.open_containing('.').working_tree()
         for path, file_class, kind, file_id in tree.list_files():
             if file_class != 'I':
                 continue
@@ -858,7 +860,7 @@
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print find_branch('.').lookup_revision(revno)
+        print Branch.open_containing('.').get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -877,13 +879,13 @@
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if revision is None:
             rev_id = b.last_patch()
         else:
             if len(revision) != 1:
                 raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
+            rev_id = revision[0].in_history(b).rev_id
         t = b.revision_tree(rev_id)
         root, ext = os.path.splitext(dest)
         if not format:
@@ -905,12 +907,12 @@
     takes_args = ['filename']
 
     def run(self, filename, revision=None):
-        if revision == None:
+        if revision is None:
             raise BzrCommandError("bzr cat requires a revision number")
         elif len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
+        b = Branch.open_containing('.')
+        b.print_file(b.relpath(filename), revision[0].in_history(b).revno)
 
 
 class cmd_local_time_offset(Command):
@@ -951,7 +953,7 @@
         from bzrlib.status import show_status
         from cStringIO import StringIO
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if selected_list:
             selected_list = [b.relpath(s) for s in selected_list]
             
@@ -993,7 +995,7 @@
     def run(self, dir='.'):
         from bzrlib.check import check
 
-        check(find_branch(dir))
+        check(Branch.open_containing(dir))
 
 
 class cmd_scan_cache(Command):
@@ -1026,7 +1028,7 @@
 
     def run(self, dir='.'):
         from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
+        upgrade(Branch.open_containing(dir))
 
 
 
@@ -1036,7 +1038,7 @@
     
     def run(self, email=False):
         try:
-            b = bzrlib.branch.find_branch('.')
+            b = bzrlib.branch.Branch.open_containing('.')
         except:
             b = None
         
@@ -1108,8 +1110,8 @@
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = find_branch(branch)
-        branch2 = find_branch(other)
+        branch1 = Branch.open_containing(branch)
+        branch2 = Branch.open_containing(other)
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
@@ -1178,15 +1180,15 @@
             other = [branch, -1]
         else:
             if len(revision) == 1:
-                other = [branch, revision[0]]
                 base = [None, None]
+                other = [branch, revision[0].in_history(branch).revno]
             else:
                 assert len(revision) == 2
                 if None in revision:
                     raise BzrCommandError(
                         "Merge doesn't permit that revision specifier.")
-                base = [branch, revision[0]]
-                other = [branch, revision[1]]
+                base = [branch, revision[0].in_history(branch).revno]
+                other = [branch, revision[1].in_history(branch).revno]
 
         try:
             merge(other, base, check_clean=(not force), merge_type=merge_type)
@@ -1229,7 +1231,7 @@
               backup_files=not no_backup,
               file_list=file_list)
         if not file_list:
-            Branch('.').set_pending_merges([])
+            Branch.open_containing('.').set_pending_merges([])
 
 
 class cmd_assert_fail(Command):
@@ -1283,7 +1285,7 @@
         if verbose and quiet:
             raise BzrCommandError('Cannot pass both quiet and verbose')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         parent = b.get_parent()
         if remote is None:
             if parent is None:
@@ -1293,14 +1295,13 @@
                     print "Using last location: %s" % parent
                 remote = parent
         elif parent is None:
-            # We only update parent if it did not exist, missing should not change the parent
+            # We only update parent if it did not exist, missing
+            # should not change the parent
             b.set_parent(remote)
-        br_remote = find_branch(remote)
-
+        br_remote = Branch.open_containing(remote)
         return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
 
 
-
 class cmd_plugins(Command):
     """List plugins"""
     hidden = True

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-16 14:40:41 +0000
+++ bzrlib/commands.py	2005-09-16 14:57:59 +0000
@@ -42,7 +42,7 @@
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
-from bzrlib.branch import find_branch
+from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
 
 plugin_cmds = {}
@@ -76,70 +76,68 @@
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
 
-    This always returns a list.  The list will have one element for 
-
-    It supports integers directly, but everything else it
-    defers for passing to Branch.get_revision_info()
+    This always returns a list.  The list will have one element for
+    each revision.
 
     >>> _parse_revision_str('234')
-    [234]
+    [<RevisionSpec_int 234>]
     >>> _parse_revision_str('234..567')
-    [234, 567]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
     >>> _parse_revision_str('..')
-    [None, None]
+    [<RevisionSpec None>, <RevisionSpec None>]
     >>> _parse_revision_str('..234')
-    [None, 234]
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
     >>> _parse_revision_str('234..')
-    [234, None]
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
     >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [234, 456, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('234....789') # Error?
-    [234, None, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('revid:test@other.com-234234')
-    ['revid:test@other.com-234234']
+    [<RevisionSpec_revid revid:test@other.com-234234>]
     >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    ['revid:test@other.com-234234', 'revid:test@other.com-234235']
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
     >>> _parse_revision_str('revid:test@other.com-234234..23')
-    ['revid:test@other.com-234234', 23]
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('date:2005-04-12')
-    ['date:2005-04-12']
+    [<RevisionSpec_date date:2005-04-12>]
     >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    ['date:2005-04-12 12:24:33']
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
     >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    ['date:2005-04-12T12:24:33']
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
     >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    ['date:2005-04-12,12:24:33']
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
     >>> _parse_revision_str('-5..23')
-    [-5, 23]
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('-5')
-    [-5]
+    [<RevisionSpec_int -5>]
     >>> _parse_revision_str('123a')
-    ['123a']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
     >>> _parse_revision_str('abc')
-    ['abc']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
     """
     import re
     old_format_re = re.compile('\d*:\d*')
     m = old_format_re.match(revstr)
+    revs = []
     if m:
         warning('Colon separator for revision numbers is deprecated.'
                 ' Use .. instead')
-        revs = []
         for rev in revstr.split(':'):
             if rev:
-                revs.append(int(rev))
-            else:
-                revs.append(None)
-        return revs
-    revs = []
-    for x in revstr.split('..'):
-        if not x:
-            revs.append(None)
-        else:
-            try:
-                revs.append(int(x))
-            except ValueError:
-                revs.append(x)
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            else:
+                revs.append(RevisionSpec(x))
     return revs
 
 
@@ -408,13 +406,13 @@
     >>> parse_args('commit --message=biter'.split())
     (['commit'], {'message': u'biter'})
     >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [500]})
+    (['log'], {'revision': [<RevisionSpec_int 500>]})
     >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [500, 600]})
+    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
     >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [500, 600]})
-    >>> parse_args('log -rv500..600'.split()) #the r takes an argument
-    (['log'], {'revision': ['v500', 600]})
+    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
     args = []
     opts = {}

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-16 14:40:41 +0000
+++ bzrlib/commit.py	2005-09-16 14:57:59 +0000
@@ -57,7 +57,7 @@
         If null (default), a time/random revision id is generated.
     """
 
-    import time, tempfile
+    import time, tempfile, re
 
     from bzrlib.osutils import local_time_offset, username
     from bzrlib.branch import gen_file_id
@@ -144,6 +144,22 @@
             timezone = local_time_offset()
 
         mutter("building commit log message")
+        # Python strings can include characters that can't be
+        # represented in well-formed XML; escape characters that
+        # aren't listed in the XML specification
+        # (http://www.w3.org/TR/REC-xml/#NT-Char).
+        if isinstance(message, unicode):
+            char_pattern = u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]'
+        else:
+            # Use a regular 'str' as pattern to avoid having re.subn
+            # return 'unicode' results.
+            char_pattern = '[^x09\x0A\x0D\x20-\xFF]'
+        message, escape_count = re.subn(
+            char_pattern,
+            lambda match: match.group(0).encode('unicode_escape'),
+            message)
+        if escape_count:
+            note("replaced %d control characters in message", escape_count)
         rev = Revision(timestamp=timestamp,
                        timezone=timezone,
                        committer=committer,

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-09-16 14:40:41 +0000
+++ bzrlib/diff.py	2005-09-16 14:57:59 +0000
@@ -161,15 +161,15 @@
         import sys
         output = sys.stdout
 
-    if revision == None:
+    if revision is None:
         old_tree = b.basis_tree()
     else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
+        old_tree = b.revision_tree(revision.in_history(b).rev_id)
 
-    if revision2 == None:
+    if revision2 is None:
         new_tree = b.working_tree()
     else:
-        new_tree = b.revision_tree(b.lookup_revision(revision2))
+        new_tree = b.revision_tree(revision2.in_branch(b).rev_id)
 
     show_diff_trees(old_tree, new_tree, output, specific_files,
                     external_diff_options)

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-16 14:40:41 +0000
+++ bzrlib/help.py	2005-09-16 14:57:59 +0000
@@ -30,24 +30,24 @@
 
 Basic commands:
 
-  bzr init      makes this branch versioned
-  bzr branch    make a copy of another branch
-
-  bzr add       make files or directories versioned
-  bzr ignore    ignore a file or pattern
-  bzr mv        move or rename a versioned file
-
-  bzr status    summarize changes in working copy
-  bzr diff      show detailed diffs
-
-  bzr merge     pull in changes from another branch
-  bzr commit    save some or all changes
-
-  bzr log       show history of changes
-  bzr check     validate storage
-
-Use e.g. 'bzr help init' for more details, or
-'bzr help commands' for all commands.
+  bzr init           makes this directory a versioned branch
+  bzr branch         make a copy of another branch
+
+  bzr add            make files or directories versioned
+  bzr ignore         ignore a file or pattern
+  bzr mv             move or rename a versioned file
+
+  bzr status         summarize changes in working copy
+  bzr diff           show detailed diffs
+
+  bzr merge          pull in changes from another branch
+  bzr commit         save some or all changes
+
+  bzr log            show history of changes
+  bzr check          validate storage
+
+  bzr help init      more help on e.g. init command
+  bzr help commands  list all commands
 """
 
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-09-16 14:40:41 +0000
+++ bzrlib/inventory.py	2005-09-16 14:57:59 +0000
@@ -254,7 +254,7 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch(init=True) create a unique inventory
+        # We are letting Branch.initialize() create a unique inventory
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-16 14:40:41 +0000
+++ bzrlib/merge.py	2005-09-16 14:57:59 +0000
@@ -26,7 +26,7 @@
 from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
 from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
 from bzrlib.changeset import Inventory, Diff3Merge
-from bzrlib.branch import find_branch
+from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
 from bzrlib.errors import NoCommits
 from bzrlib.delta import compare_trees
@@ -134,13 +134,13 @@
             
 def get_tree(treespec, temp_root, label, local_branch=None):
     location, revno = treespec
-    branch = find_branch(location)
+    branch = Branch.open(location)
     if revno is None:
         revision = None
     elif revno == -1:
         revision = branch.last_patch()
     else:
-        revision = branch.lookup_revision(revno)
+        revision = branch.get_rev_id(revno)
     return branch, get_revid_tree(branch, revision, temp_root, label,
                                   local_branch)
 
@@ -244,7 +244,7 @@
     try:
         if this_dir is None:
             this_dir = '.'
-        this_branch = find_branch(this_dir)
+        this_branch = Branch.open_containing(this_dir)
         this_rev_id = this_branch.last_patch()
         if this_rev_id is None:
             raise BzrCommandError("This branch has no commits")
@@ -261,7 +261,7 @@
                 raise NoCommits(other_branch)
             other_basis = other_rev_id
         elif other_revision[1] is not None:
-            other_rev_id = other_branch.lookup_revision(other_revision[1])
+            other_rev_id = other_branch.get_rev_id(other_revision[1])
             other_basis = other_rev_id
         else:
             other_rev_id = None
@@ -284,7 +284,7 @@
             elif base_revision[1] is None:
                 base_rev_id = None
             else:
-                base_rev_id = base_branch.lookup_revision(base_revision[1])
+                base_rev_id = base_branch.get_rev_id(base_revision[1])
             if base_rev_id is not None:
                 base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
                                                MultipleRevisionSources(this_branch, 

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-16 14:40:41 +0000
+++ bzrlib/remotebranch.py	2005-09-16 14:57:59 +0000
@@ -29,7 +29,7 @@
 import urlparse
 
 from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.branch import Branch, LocalBranch, BZR_BRANCH_FORMAT
 from bzrlib.trace import mutter
 from bzrlib.xml import serializer_v4
 
@@ -110,7 +110,7 @@
         
 
 
-class RemoteBranch(Branch):
+class RemoteBranch(LocalBranch):
     def __init__(self, baseurl, find_root=True):
         """Create new proxy for a remote branch."""
         if find_root:
@@ -129,6 +129,16 @@
 
     __repr__ = __str__
 
+    def setup_caching(self, cache_root):
+        """Set up cached stores located under cache_root"""
+        from bzrlib.meta_store import CachedStore
+        for store_name in ('inventory_store', 'text_store', 'revision_store'):
+            if not isinstance(getattr(self, store_name), CachedStore):
+                cache_path = os.path.join(cache_root, store_name)
+                os.mkdir(cache_path)
+                new_store = CachedStore(getattr(self, store_name), cache_path)
+                setattr(self, store_name, new_store)
+
     def controlfile(self, filename, mode):
         if mode not in ('rb', 'rt', 'r'):
             raise BzrError("file mode %r not supported for remote branches" % mode)

=== added file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revisionspec.py	2005-09-16 14:57:59 +0000
@@ -0,0 +1,334 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import datetime
+import re
+from bzrlib.errors import BzrError, NoSuchRevision, NoCommits
+
+_marker = []
+
+class RevisionInfo(object):
+    """The results of applying a revision specification to a branch.
+
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    This class exists mostly to be the return value of a RevisionSpec,
+    so that you can access the member you're interested in (number or id)
+    or treat the result as a tuple.
+    """
+
+    def __init__(self, branch, revno, rev_id=_marker):
+        self.branch = branch
+        self.revno = revno
+        if rev_id is _marker:
+            # allow caller to be lazy
+            if self.revno is None:
+                self.rev_id = None
+            else:
+                self.rev_id = branch.get_rev_id(self.revno)
+        else:
+            self.rev_id = rev_id
+
+    def __nonzero__(self):
+        # first the easy ones...
+        if self.rev_id is None:
+            return False
+        if self.revno is not None:
+            return True
+        # TODO: otherwise, it should depend on how I was built -
+        # if it's in_history(branch), then check revision_history(),
+        # if it's in_store(branch), do the check below
+        return self.rev_id in self.branch.revision_store
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
+
+# classes in this list should have a "prefix" attribute, against which
+# string specs are matched
+SPEC_TYPES = []
+
+class RevisionSpec(object):
+    """A parsed revision specification.
+
+    A revision specification can be an integer, in which case it is
+    assumed to be a revno (though this will translate negative values
+    into positive ones); or it can be a string, in which case it is
+    parsed for something like 'date:' or 'revid:' etc.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
+
+    (Equivalent to the old Branch method get_revision_info())
+    """
+
+    prefix = None
+
+    def __new__(cls, spec, foo=_marker):
+        """Parse a revision specifier.
+        """
+        if spec is None:
+            return object.__new__(RevisionSpec, spec)
+
+        try:
+            spec = int(spec)
+        except ValueError:
+            pass
+
+        if isinstance(spec, int):
+            return object.__new__(RevisionSpec_int, spec)
+        elif isinstance(spec, basestring):
+            for spectype in SPEC_TYPES:
+                if spec.startswith(spectype.prefix):
+                    return object.__new__(spectype, spec)
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+    def __init__(self, spec):
+        if self.prefix and spec.startswith(self.prefix):
+            spec = spec[len(self.prefix):]
+        self.spec = spec
+
+    def _match_on(self, branch, revs):
+        return RevisionInfo(branch, 0, None)
+
+    def _match_on_and_check(self, branch, revs):
+        info = self._match_on(branch, revs)
+        if info:
+            return info
+        elif info == (0, None):
+            # special case - the empty tree
+            return info
+        elif self.prefix:
+            raise NoSuchRevision(branch, self.prefix + str(self.spec))
+        else:
+            raise NoSuchRevision(branch, str(self.spec))
+
+    def in_history(self, branch):
+        revs = branch.revision_history()
+        return self._match_on_and_check(branch, revs)
+
+    def __repr__(self):
+        # this is mostly for helping with testing
+        return '<%s %s%s>' % (self.__class__.__name__,
+                              self.prefix or '',
+                              self.spec)
+
+
+# private API
+
+class RevisionSpec_int(RevisionSpec):
+    """Spec is a number.  Special case."""
+    def __init__(self, spec):
+        self.spec = int(spec)
+
+    def _match_on(self, branch, revs):
+        if self.spec < 0:
+            revno = len(revs) + self.spec + 1
+        else:
+            revno = self.spec
+        rev_id = branch.get_rev_id(revno, revs)
+        return RevisionInfo(branch, revno, rev_id)
+
+
+class RevisionSpec_revno(RevisionSpec):
+    prefix = 'revno:'
+
+    def _match_on(self, branch, revs):
+        """Lookup a revision by revision number"""
+        try:
+            return RevisionInfo(branch, int(self.spec))
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revno)
+
+
+class RevisionSpec_revid(RevisionSpec):
+    prefix = 'revid:'
+
+    def _match_on(self, branch, revs):
+        try:
+            return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revid)
+
+
+class RevisionSpec_last(RevisionSpec):
+    prefix = 'last:'
+
+    def _match_on(self, branch, revs):
+        try:
+            offset = int(self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+        else:
+            if offset <= 0:
+                raise BzrError('You must supply a positive value for --revision last:XXX')
+            return RevisionInfo(branch, len(revs) - offset + 1)
+
+SPEC_TYPES.append(RevisionSpec_last)
+
+
+class RevisionSpec_tag(RevisionSpec):
+    prefix = 'tag:'
+
+    def _match_on(self, branch, revs):
+        raise BzrError('tag: namespace registered, but not implemented.')
+
+SPEC_TYPES.append(RevisionSpec_tag)
+
+
+class RevisionSpec_date(RevisionSpec):
+    prefix = 'date:'
+    _date_re = re.compile(
+            r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+            r'(,|T)?\s*'
+            r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+        )
+
+    def _match_on(self, branch, revs):
+        """
+        Spec for date revisions:
+          date:value
+          value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+          it can also start with a '+/-/='. '+' says match the first
+          entry after the given date. '-' is match the first entry before the date
+          '=' is match the first entry after, but still on the given date.
+
+          +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
+          -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
+          =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
+              May 13th, 2005 at 0:00
+
+          So the proper way of saying 'give me all entries for today' is:
+              -r {date:+today}:{date:-tomorrow}
+          The default is '=' when not supplied
+        """
+        match_style = '='
+        if self.spec[:1] in ('+', '-', '='):
+            match_style = self.spec[:1]
+            self.spec = self.spec[1:]
+
+        # XXX: this should probably be using datetime.date instead
+        today = datetime.datetime.today().replace(hour=0, minute=0, second=0,
+                                                  microsecond=0)
+        if self.spec.lower() == 'yesterday':
+            dt = today - datetime.timedelta(days=1)
+        elif self.spec.lower() == 'today':
+            dt = today
+        elif self.spec.lower() == 'tomorrow':
+            dt = today + datetime.timedelta(days=1)
+        else:
+            m = self._date_re.match(self.spec)
+            if not m or (not m.group('date') and not m.group('time')):
+                raise BzrError('Invalid revision date %r' % self.spec)
+
+            if m.group('date'):
+                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
+            else:
+                year, month, day = today.year, today.month, today.day
+            if m.group('time'):
+                hour = int(m.group('hour'))
+                minute = int(m.group('minute'))
+                if m.group('second'):
+                    second = int(m.group('second'))
+                else:
+                    second = 0
+            else:
+                hour, minute, second = 0,0,0
+
+            dt = datetime.datetime(year=year, month=month, day=day,
+                    hour=hour, minute=minute, second=second)
+        first = dt
+        last = None
+        reversed = False
+        if match_style == '-':
+            reversed = True
+        elif match_style == '=':
+            last = dt + datetime.timedelta(days=1)
+
+        if reversed:
+            for i in range(len(revs)-1, -1, -1):
+                r = branch.get_revision(revs[i])
+                # TODO: Handle timezone.
+                dt = datetime.datetime.fromtimestamp(r.timestamp)
+                if first >= dt and (last is None or dt >= last):
+                    return RevisionInfo(branch, i+1,)
+        else:
+            for i in range(len(revs)):
+                r = branch.get_revision(revs[i])
+                # TODO: Handle timezone.
+                dt = datetime.datetime.fromtimestamp(r.timestamp)
+                if first <= dt and (last is None or dt <= last):
+                    return RevisionInfo(branch, i+1,)
+
+SPEC_TYPES.append(RevisionSpec_date)
+
+
+class RevisionSpec_ancestor(RevisionSpec):
+    prefix = 'ancestor:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from revision import common_ancestor, MultipleRevisionSources
+        other_branch = Branch.open_containing(self.spec)
+        revision_a = branch.last_patch()
+        revision_b = other_branch.last_patch()
+        for r, b in ((revision_a, branch), (revision_b, other_branch)):
+            if r is None:
+                raise NoCommits(b)
+        revision_source = MultipleRevisionSources(branch, other_branch)
+        rev_id = common_ancestor(revision_a, revision_b, revision_source)
+        try:
+            revno = branch.revision_id_to_revno(rev_id)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, rev_id)
+        
+SPEC_TYPES.append(RevisionSpec_ancestor)

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/__init__.py	2005-09-16 14:57:59 +0000
@@ -20,9 +20,11 @@
 import tempfile
 import os
 import sys
+import errno
 import subprocess
+import shutil
 
-from testsweet import run_suite
+import testsweet
 import bzrlib.commands
 
 import bzrlib.trace
@@ -188,17 +190,22 @@
             self.fail("contents of %s not as expected")
 
     def _make_test_root(self):
-        import os
-        import shutil
-        import tempfile
-        
         if TestCaseInTempDir.TEST_ROOT is not None:
             return
-        TestCaseInTempDir.TEST_ROOT = os.path.abspath(
-                                 tempfile.mkdtemp(suffix='.tmp',
-                                                  prefix=self._TEST_NAME + '-',
-                                                  dir=os.curdir))
-    
+        i = 0
+        while True:
+            root = 'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            break
         # make a fake bzr directory there to prevent any tests propagating
         # up onto the source directory's real branch
         os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
@@ -292,7 +299,7 @@
 
 
 def selftest(verbose=False, pattern=".*"):
-    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+    return testsweet.run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
 
 
 def test_suite():

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-16 14:57:59 +0000
@@ -93,7 +93,7 @@
     def test_ignore_patterns(self):
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(list(b.unknowns()), [])
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
@@ -150,7 +150,7 @@
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         from bzrlib.branch import Branch
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['a', 'c', 'subdir/'])
         self.run_bzr('add', self.test_dir)
         self.run_bzr('mv', 'a', 'b')
@@ -214,8 +214,8 @@
         self.runbzr('merge ../b')
         self.check_file_contents('goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
-        a = Branch('.')
-        b = Branch('../b')
+        a = Branch.open('.')
+        b = Branch.open('../b')
         a.get_revision_xml(b.last_patch())
         self.log('pending merges: %s', a.pending_merges())
         #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
@@ -241,8 +241,8 @@
         self.runbzr('pull')
         self.runbzr('commit -m blah --unchanged')
         os.chdir('../a')
-        a = Branch('.')
-        b = Branch('../b')
+        a = Branch.open('.')
+        b = Branch.open('../b')
         assert a.revision_history() == b.revision_history()[:-1]
         self.runbzr('pull ../b')
         assert a.revision_history() == b.revision_history()
@@ -260,7 +260,7 @@
 
     def test_add_reports(self):
         """add command prints the names of added files."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
 
         from cStringIO import StringIO

=== modified file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-09-16 14:57:59 +0000
@@ -475,7 +475,7 @@
         from bzrlib.merge import merge
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
-        branch = Branch("original", init=True)
+        branch = Branch.initialize("original")
         smart_add_branch(branch, ["original"], True, add_reporter_null)
         branch.commit("start branch.", verbose=False)
         # Mary branches it.
@@ -487,7 +487,7 @@
         file.close()
         branch.commit("change file1")
         # Mary does too
-        mary_branch = Branch("mary")
+        mary_branch = Branch.open("mary")
         file = open("mary/file2", "wt")
         file.write("Mary\n")
         file.close()

=== modified file 'bzrlib/selftest/test_parent.py'
--- bzrlib/selftest/test_parent.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/test_parent.py	2005-09-16 14:57:59 +0000
@@ -24,13 +24,13 @@
 class TestParent(TestCaseInTempDir):
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(b.get_parent(), None)
         
     
     def test_set_get_parent(self):
         """Set and then re-get the parent"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         url = 'http://bazaar-ng.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEquals(b.get_parent(), url)
@@ -40,7 +40,7 @@
         from bzrlib.commands import run_bzr
 
         os.mkdir('from')
-        branch_from = Branch('from', init=True)
+        branch_from = Branch.initialize('from')
         file('from/foo', 'wt').write('contents of foo')
         branch_from.add('foo')
         branch_from.commit('initial commit')
@@ -48,7 +48,7 @@
         os.mkdir('to')
         copy_branch(branch_from, 'to', None)
 
-        branch_to = Branch('to')
+        branch_to = Branch.open('to')
         abspath = os.path.abspath('from')
         self.assertEquals(branch_to.get_parent(), abspath)
         

=== modified file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-09-16 14:57:59 +0000
@@ -12,7 +12,7 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add((".",), recurse=True)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -22,7 +22,7 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add((".",), recurse=True)
         for path in paths:
@@ -35,7 +35,7 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add(("branch",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -51,8 +51,8 @@
                        "original/child/", "original/child/path")
         
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add((".",), True, add_reporter_null)
         for path in paths:
             self.assertNotEqual((path, branch.inventory.path2id(path)),
@@ -68,7 +68,7 @@
         from bzrlib.add import smart_add
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add(paths)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -81,7 +81,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, (".",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -91,7 +91,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add_branch(branch, (".",))
         for path in paths:
@@ -104,7 +104,7 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add_branch(branch, ("branch",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -118,8 +118,8 @@
         build_paths = ("original/", "original/file1", "original/file2", 
                        "original/child/", "original/child/path")
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add_branch(branch, (".",))
         for path in paths:
             self.assertNotEqual((path, branch.inventory.path2id(path)),
@@ -135,7 +135,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, paths)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testbranch.py	2005-09-16 14:57:59 +0000
@@ -21,7 +21,7 @@
     """Test appending more than one revision"""
     def test_append_revisions(self):
         from bzrlib.branch import Branch
-        br = Branch(".", init=True)
+        br = Branch.initialize(".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")

=== modified file 'bzrlib/selftest/testfetch.py'
--- bzrlib/selftest/testfetch.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testfetch.py	2005-09-16 14:57:59 +0000
@@ -29,7 +29,7 @@
 
         def new_branch(name):
             os.mkdir(name)
-            return Branch(name, init=True)
+            return Branch.initialize(name)
             
         #highest indices a: 5, b: 7
         br_a, br_b = make_branches()

=== modified file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testlog.py	2005-09-16 14:57:59 +0000
@@ -83,10 +83,29 @@
         self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
                           start_revision=1, end_revision=-1) 
 
+    def test_cur_revno(self):
+        b = Branch.initialize('.')
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
     def test_simple_log(self):
         eq = self.assertEquals
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         lf = LogCatcher()
         show_log(b, lf)
@@ -132,3 +151,12 @@
         self.log('log 2 delta: %r' % d)
         # self.checkDelta(d, added=['hello'])
         
+        # commit a log message with control characters
+        msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
+        b.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg != committed_msg)
+        self.assert_(len(committed_msg) > len(msg))

=== modified file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-16 14:57:59 +0000
@@ -7,7 +7,7 @@
 class TestMerge(TestCaseInTempDir):
     """Test appending more than one revision"""
     def test_pending(self):
-        br = Branch(".", init=True)
+        br = Branch.initialize(".")
         commit(br, "lala!")
         self.assertEquals(len(br.pending_merges()), 0)
         merge(['.', -1], [None, None])
@@ -16,7 +16,7 @@
     def test_nocommits(self):
         self.test_pending()
         os.mkdir('branch2')
-        br2 = Branch('branch2', init=True)
+        br2 = Branch.initialize('branch2')
         self.assertRaises(NoCommits, merge, ['branch2', -1], 
                           [None, None])
         return br2

=== modified file 'bzrlib/selftest/testremotebranch.py'
--- bzrlib/selftest/testremotebranch.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testremotebranch.py	2005-09-16 14:57:59 +0000
@@ -29,11 +29,11 @@
 
         def new_branch(name):
             os.mkdir(name)
-            return Branch(name, init=True)
+            return Branch.initialize(name)
             
         #highest indices a: 5, b: 7
         br_a, br_b = make_branches()
-        br_rem = find_branch(self.get_remote_url(br_a.base))
+        br_rem = Branch.open(self.get_remote_url(br_a.base))
         assert not has_revision(br_b, br_rem.revision_history()[3])
         assert has_revision(br_b, br_rem.revision_history()[2])
         assert len(br_b.revision_history()) == 7

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testrevision.py	2005-09-16 14:57:59 +0000
@@ -22,14 +22,14 @@
     from bzrlib.commit import commit
     import os
     os.mkdir("branch1")
-    br1 = Branch("branch1", init=True)
+    br1 = Branch.initialize("branch1")
     
     commit(br1, "Commit one", rev_id="a@u-0-0")
     commit(br1, "Commit two", rev_id="a@u-0-1")
     commit(br1, "Commit three", rev_id="a@u-0-2")
 
     os.mkdir("branch2")
-    br2 = Branch("branch2", init=True)
+    br2 = Branch.initialize("branch2")
     br2.update_revisions(br1)
     commit(br2, "Commit four", rev_id="b@u-0-3")
     commit(br2, "Commit five", rev_id="b@u-0-4")
@@ -128,6 +128,68 @@
         self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
                           'c@u-0-6', self.br2.revision_history())
 
+class TestIntermediateRevisions(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseInTempDir.setUp(self)
+        self.br1, self.br2 = make_branches()
+        commit(self.br2, "Commit eleven", rev_id="b@u-0-7")
+        commit(self.br2, "Commit twelve", rev_id="b@u-0-8")
+        commit(self.br2, "Commit thirtteen", rev_id="b@u-0-9")
+        self.br1.add_pending_merge(self.br2.revision_history()[6])
+        commit(self.br1, "Commit fourtten", rev_id="a@u-0-6")
+        self.br2.add_pending_merge(self.br1.revision_history()[6])
+        commit(self.br2, "Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1, self.br2)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
 
 class TestCommonAncestor(TestCaseInTempDir):
     """Test checking whether a revision is an ancestor of another revision"""

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-16 14:57:59 +0000
@@ -22,40 +22,55 @@
 
 class TestRevisionNamespaces(TestCaseInTempDir):
     def test_revision_namespaces(self):
-        """Functional tests for hashcache"""
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
         from bzrlib.errors import NoSuchRevision
         from bzrlib.branch import Branch
+        from bzrlib.revisionspec import RevisionSpec
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         b.commit('Commit one', rev_id='a@r-0-1')
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
-        self.assertEquals(b.get_revision_info(None), (0, None))
-        self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(NoSuchRevision, b.get_revision_info, 'revid:a@r-0-0')
-        self.assertRaises(TypeError, b.get_revision_info, object)
-
-        self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
+        self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revid:a@r-0-1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision,
+                          RevisionSpec('revid:a@r-0-0').in_history, b)
+        self.assertRaises(TypeError, RevisionSpec, object)
+
+        self.assertEquals(RevisionSpec('date:-tomorrow').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('date:+today').in_history(b),
+                          (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec('last:1').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('-1').in_history(b), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b).rev_id,
+                          'a@r-0-3')
 
         os.mkdir('newbranch')
-        b2 = Branch('newbranch', init=True)
-        self.assertEquals(b2.lookup_revision('revid:a@r-0-1'), 'a@r-0-1')
+        b2 = Branch.initialize('newbranch')
+        self.assertRaises(NoCommits, RevisionSpec('ancestor:.').in_history, b2)
 
-        self.assertRaises(NoCommits, b2.lookup_revision, 'ancestor:.')
-        self.assertEquals(b.lookup_revision('ancestor:.'), 'a@r-0-3')
         os.mkdir('copy')
         b3 = copy_branch(b, 'copy')
         b3.commit('Commit four', rev_id='b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'a@r-0-3')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'a@r-0-3')
         merge(['copy', -1], [None, None])
         b.commit('Commit five', rev_id='a@r-0-4')
-        self.assertEquals(b.lookup_revision('ancestor:copy'), 'b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
+                          'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'b@r-0-4')

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/teststatus.py	2005-09-16 14:57:59 +0000
@@ -31,7 +31,7 @@
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         # status with nothing
         tof = StringIO()

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/versioning.py	2005-09-16 14:57:59 +0000
@@ -44,7 +44,7 @@
 
         from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
-        b = Branch('.')
+        b = Branch.open('.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
@@ -63,7 +63,7 @@
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['foo/',
                          'foo/hello'])
@@ -84,7 +84,7 @@
         from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
@@ -112,7 +112,7 @@
         from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
@@ -131,7 +131,7 @@
         ass = self.assert_
         chdir = os.chdir
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['src/', 'README'])
         
         eq(sorted(b.unknowns()),
@@ -159,7 +159,7 @@
         """After all the above changes, run the check and upgrade commands.
 
         The upgrade should be a no-op."""
-        b = Branch('.')
+        b = Branch.open('.')
         debug('branch has %d revisions', b.revno())
         
         debug('check branch...')
@@ -186,7 +186,7 @@
         self.build_tree(['a/', 'b/'])
         
         run_bzr('init')
-        b = Branch('.')
+        b = Branch.open('.')
         
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('old contents')
@@ -199,8 +199,8 @@
             
         run_bzr('commit', 'a', '-m', 'commit a only')
         
-        old = b.revision_tree(b.lookup_revision(1))
-        new = b.revision_tree(b.lookup_revision(2))
+        old = b.revision_tree(b.get_rev_id(1))
+        new = b.revision_tree(b.get_rev_id(2))
         
         eq(new.get_file_by_path('b/two').read(), 'old contents')
         eq(new.get_file_by_path('top').read(), 'old contents')
@@ -209,14 +209,14 @@
         os.chdir('a')
         # commit from here should do nothing
         run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
-        v3 = b.revision_tree(b.lookup_revision(3))
+        v3 = b.revision_tree(b.get_rev_id(3))
         eq(v3.get_file_by_path('b/two').read(), 'old contents')
         eq(v3.get_file_by_path('top').read(), 'old contents')
         eq(v3.get_file_by_path('a/one').read(), 'new contents')
                 
         # commit in subdirectory commits whole tree
         run_bzr('commit', '-m', 'commit whole tree from subdir')
-        v4 = b.revision_tree(b.lookup_revision(4))
+        v4 = b.revision_tree(b.get_rev_id(4))
         eq(v4.get_file_by_path('b/two').read(), 'new contents')        
         eq(v4.get_file_by_path('top').read(), 'new contents')
         

=== modified file 'bzrlib/selftest/whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-09-16 14:40:41 +0000
+++ bzrlib/selftest/whitebox.py	2005-09-16 14:57:59 +0000
@@ -9,7 +9,7 @@
 class TestBranch(TestCaseInTempDir):
 
     def test_unknowns(self):
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt',
                          'hello.txt~'])
@@ -20,7 +20,7 @@
     def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
 
@@ -74,7 +74,7 @@
 
     def test_pending_merges(self):
         """Tracking pending-merged revisions."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.assertEquals(b.pending_merges(), [])
         b.add_pending_merge('foo@azkhazan-123123-abcabc')
@@ -97,7 +97,7 @@
         
     def test_revert(self):
         """Test selected-file revert"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
@@ -124,7 +124,7 @@
 
     def test_rename_dirs(self):
         """Test renaming directories and the files within them."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
         b.add(['dir', 'dir/sub', 'dir/sub/file'])
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-09-16 14:40:41 +0000
+++ bzrlib/transport/__init__.py	2005-09-16 14:57:59 +0000
@@ -92,7 +92,7 @@
 
     def clone(self, offset=None):
         """Return a new Transport object, cloned from the current location,
-        using a subdirectory. This allows connections to be pooled,
+        using a subdirectory or parent directory. This allows connections to be pooled,
         rather than a new one needed for each subdir.
         """
         raise NotImplementedError

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/http.py	2005-09-16 14:57:59 +0000
@@ -118,7 +118,9 @@
         if isinstance(relpath, basestring):
             relpath = [relpath]
         baseurl = self.base.rstrip('/')
-        return '/'.join([baseurl] + relpath)
+        # TODO: We should handle ".." in a more appropriate manner
+        # Basically, we need to normalize the path.
+        path = '/'.join([baseurl] + relpath)
 
     def relpath(self, abspath):
         if not abspath.startswith(self.base):

